package logic

import (
	"errors"
	"fmt"
	"fusenapi/model/gmodel"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"sort"

	"context"

	"fusenapi/server/ldap-admin/internal/svc"
	"fusenapi/server/ldap-admin/internal/types"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetDepartmentsLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetDepartmentsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetDepartmentsLogic {
	return &GetDepartmentsLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetDepartmentsLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

func (l *GetDepartmentsLogic) GetDepartments(req *types.Request, userinfo *auth.UserInfo) (resp *basic.Response) {
	//todo 鉴权 。。。。
	departList, _, err := l.svcCtx.AllModels.LdapDepartment.GetAll(l.ctx, "sort ASC")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "获取部门列表失败")
	}
	//变成树形结构
	list, err := l.DepartmentListToTree(departList, false)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeServiceErr, err.Error())
	}
	fmt.Println(l.SyncDepartmentToLdap())
	return resp.SetStatusWithMessage(basic.CodeOK, "success", types.GetDepartmentsRsp{
		List: list,
	})
}

// 把列表变成树形结构
func (l *GetDepartmentsLogic) DepartmentListToTree(deps []gmodel.LdapDepartment, withDepMember bool) ([]*types.DepartmentsItem, error) {
	var (
		ldapUserList []gmodel.GetAllUserWithDepartmentRsp
		err          error
	)
	if withDepMember {
		ldapUserList, err = l.svcCtx.AllModels.LdapUsers.GetAllUserWithDepartment(l.ctx)
		if err != nil {
			logx.Error(err)
			return nil, errors.New("获取全部部门用户失败")
		}
	}
	//存入map
	mapDepartment := make(map[int64]*types.DepartmentsItem)
	for _, v := range deps {
		data := &types.DepartmentsItem{
			Id:        v.Id,
			Name:      *v.Name,
			Remark:    *v.Remark,
			Type:      *v.Type,
			ParentId:  *v.ParentId,
			Dn:        *v.Dn,
			SyncState: *v.SyncState,
			Sort:      *v.Sort,
			Child:     make([]*types.DepartmentsItem, 0, 50),
			Members:   nil,
		}
		members := make([]types.Member, 0, 100)
		for _, user := range ldapUserList {
			if *user.DepartmentId != v.Id {
				continue
			}
			members = append(members, types.Member{
				Id:       user.Id,
				Name:     *user.Username,
				Nickname: *user.Nickname,
				Email:    *user.Email,
			})
		}
		data.Members = members
		mapDepartment[v.Id] = data
	}
	//组织从属关系
	for _, v := range mapDepartment {
		//如果有父级
		if parent, ok := mapDepartment[v.ParentId]; ok {
			parent.Child = append(parent.Child, v)
			sort.Slice(parent.Child, func(i, j int) bool {
				return parent.Child[i].Sort < parent.Child[j].Sort //升序
			})
		}
	}
	//排序
	list := make([]*types.DepartmentsItem, 0, len(deps))
	for _, v := range deps {
		if *v.ParentId == 0 {
			list = append(list, mapDepartment[v.Id])
		}
	}
	return list, nil
}

// 同步到ldap
func (l *GetDepartmentsLogic) SyncDepartmentToLdap() error {
	/*	departList, _, err := l.svcCtx.AllModels.LdapDepartment.GetAll(l.ctx, "sort ASC")
		if err != nil {
			return err
		}
		//获取所有部门用户
		ldapUserList, err := l.svcCtx.AllModels.LdapUsers.GetAllUserWithDepartment(l.ctx)
		if err != nil {
			return err
		}
		for _, v := range departList {
			err = ildap.Department.Add(l.svcCtx.Ldap, l.svcCtx.Config.Ldap.AdminDN, ildap.DepartmentData{
				Id:       v.Id,
				Name:     *v.Name,
				Remark:   *v.Remark,
				Type:     *v.Type,
				ParentId: *v.ParentId,
				Dn:       *v.Dn,
			})
			if err != nil {
				logx.Error(err)
				return errors.New("向LDAP同步分组失败")
			}
			userList := make([]gmodel.LdapUsers, 0, 100)
			for _, user := range ldapUserList {
				if *user.DepartmentId != v.Id {
					continue
				}
				userList = append(userList, user.LdapUsers)
			}
			if len(userList) > 0 {
				for _, user := range userList {
					if *user.UserDn == l.svcCtx.Config.Ldap.AdminDN {
						continue
					}
					err = ildap.Department.AddUserToGroup(l.svcCtx.Ldap, *v.Dn, *user.UserDn)
					if err != nil {
						logx.Error(err)
						return errors.New("把用户添加到ldap分组失败")
					}
				}
			}
			//更新的更新状态
			syncState := int64(1)
			err = l.svcCtx.AllModels.LdapDepartment.Update(l.ctx, v.Id, &gmodel.LdapDepartment{
				SyncState: &syncState,
			})
			if err != nil {
				logx.Error(err)
				return errors.New("更新分组同步状态失败")
			}
		}*/
	return nil
}
